'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _size2 = require('lodash/size');

var _size3 = _interopRequireDefault(_size2);

var _isFunction2 = require('lodash/isFunction');

var _isFunction3 = _interopRequireDefault(_isFunction2);

exports.defaultMergeProps = defaultMergeProps;

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

var _createHOC = require('../utils/createHOC');

var _createHOC2 = _interopRequireDefault(_createHOC);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultState = {
  loading: true,
  response: null
};

var NO_RESPONSE = {};

function defaultMergeProps(response, ownProps) {
  if (response === NO_RESPONSE) {
    return null;
  }

  var _ref = response || {},
      json = _ref.json;

  return (0, _extends3.default)({}, ownProps, json);
}

function computePathSets(props, getPathSets) {
  var pathSets = (0, _isFunction3.default)(getPathSets) ? getPathSets(props) : getPathSets;
  process.env.NODE_ENV !== 'production' ? (0, _warning2.default)(typeof pathSets !== 'undefined', '"pathSets" is undefined') : void 0;
  if (pathSets === null) {
    return null;
  }
  return pathSets || [];
}

function createHandler(getPathSets, mergeProps) {
  var mergePropsFn = mergeProps || defaultMergeProps;

  var subscription = null;

  function unsubscribe() {
    if (subscription) {
      subscription.dispose();
      subscription = null;
    }
  }

  return {
    defaultState: defaultState,
    computeProps: function computeProps(props, state) {
      var newProps = mergePropsFn(state.response, props);
      if (!newProps) {
        return null;
      }
      return newProps;
    },
    subscribe: function subscribe(falcor, props, setState) {
      var pathSets = computePathSets(props, getPathSets);
      if (pathSets === null) {
        setState({
          loading: false
        });
        return;
      }
      var hasResponse = false;
      subscription = falcor.get.apply(falcor, (0, _toConsumableArray3.default)(pathSets)).subscribe(function (response) {
        hasResponse = (0, _size3.default)((0, _get3.default)(response, 'json')) > 0;
        setState({
          loading: false,
          response: response
        });
      }, function () {}, function () {
        if (!hasResponse) {
          setState({
            loading: false,
            response: NO_RESPONSE
          });
        }
      });
    },

    unsubscribe: unsubscribe
  };
}

exports.default = function (getPathSets, mergeProps, opts) {
  return (0, _createHOC2.default)(createHandler, (0, _extends3.default)({
    getDisplayName: function getDisplayName(name) {
      return 'falcorGet(' + name + ')';
    }
  }, opts), getPathSets, mergeProps);
};
//# sourceMappingURL=falcorGet.js.map