'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _isFunction2 = require('lodash/isFunction');

var _isFunction3 = _interopRequireDefault(_isFunction2);

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

var _createHOC = require('../utils/createHOC');

var _createHOC2 = _interopRequireDefault(_createHOC);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultState = {
  loading: true
};

function computePathSetToProps(props, mapPathSetToProps) {
  var pathSetToProps = (0, _isFunction3.default)(mapPathSetToProps) ? mapPathSetToProps(props) : mapPathSetToProps;
  process.env.NODE_ENV !== 'production' ? (0, _warning2.default)(typeof pathSetToProps !== 'undefined', '"pathSetToProps" is undefined') : void 0;
  return pathSetToProps || {};
}

function createHandler(mapPathSetToProps) {
  var resolvedCount = void 0;
  var subscriptions = null;
  var resolved = {};

  function unsubscribe() {
    resolvedCount = 0;
    if (subscriptions === null) {
      return;
    }
    if (subscriptions.length > 0) {
      subscriptions.forEach(function (x) {
        return x.dispose();
      });
      subscriptions = null;
    }
  }

  return {
    defaultState: defaultState,
    computeProps: function computeProps(props) {
      return (0, _extends3.default)({}, props, resolved);
    },
    subscribe: function subscribe(falcor, props, setState) {
      var pathSetToProps = computePathSetToProps(props, mapPathSetToProps);
      var propKeys = (0, _keys2.default)(pathSetToProps);
      subscriptions = propKeys.map(function (prop) {
        var pathSet = pathSetToProps[prop];
        return falcor.getValue(pathSet).subscribe(function (value) {
          resolved[prop] = value;
        }, function () {}, function () {
          resolvedCount += 1;

          if (resolvedCount === propKeys.length) {
            setState({
              loading: false
            });
          }
        });
      });
    },

    unsubscribe: unsubscribe
  };
}

exports.default = function (mapPathSetToProps, opts) {
  return (0, _createHOC2.default)(createHandler, (0, _extends3.default)({
    getDisplayName: function getDisplayName(name) {
      return 'falcorGetValue(' + name + ')';
    }
  }, opts), mapPathSetToProps);
};
//# sourceMappingURL=falcorGetValue.js.map